<?php

namespace Drupal\file_widget_show_uploaded_time\Plugin\Field\FieldWidget;

use Drupal\file\Entity\File;
use Drupal\file\FileInterface;
use Drupal\Core\Datetime\DrupalDateTime;
use Drupal\Core\Field\Attribute\FieldWidget;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Datetime\DateFormatterInterface;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Render\ElementInfoManagerInterface;
use Drupal\file\Plugin\Field\FieldWidget\FileWidget;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Plugin implementation of the 'file_widget_show_uploaded_time' widget.
 */
#[FieldWidget(
  id: 'file_widget_show_uploaded_time',
  label: new TranslatableMarkup('File showing latest uploaded date'),
  field_types: ['file'],
)]
class FileShowUploadedTimeWidget extends FileWidget {

  /**
   * The date formatter service.
   *
   * @var \Drupal\Core\Datetime\DateFormatterInterface
   */
  protected $dateFormatter;

  /**
   * The date format entity storage.
   *
   * @var \Drupal\Core\Entity\EntityStorageInterface
   */
  protected $dateFormatStorage;

  /**
   * Constructs a new DateTimeDefaultFormatter.
   *
   * @param string $plugin_id
   *   The plugin_id for the formatter.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Field\FieldDefinitionInterface $field_definition
   *   The definition of the field to which the formatter is associated.
   * @param array $settings
   *   The formatter settings.
   * @param array $third_party_settings
   *   Third party settings.
   * @param \Drupal\Core\Datetime\DateFormatterInterface $date_formatter
   *   The date formatter service.
   * @param \Drupal\Core\Entity\EntityStorageInterface $date_format_storage
   *   The date format entity storage.
   */
  public function __construct($plugin_id, $plugin_definition, FieldDefinitionInterface $field_definition, array $settings, array $third_party_settings, ElementInfoManagerInterface $element_info, DateFormatterInterface $date_formatter, EntityStorageInterface $date_format_storage) {
    parent::__construct($plugin_id, $plugin_definition, $field_definition, $settings, $third_party_settings, $element_info);

    $this->dateFormatter = $date_formatter;
    $this->dateFormatStorage = $date_format_storage;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $plugin_id,
      $plugin_definition,
      $configuration['field_definition'],
      $configuration['settings'],
      $configuration['third_party_settings'],
      $container->get('element_info'),
      $container->get('date.formatter'),
      $container->get('entity_type.manager')->getStorage('date_format')
    );
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      'format_type' => 'fallback',
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $form = parent::settingsForm($form, $form_state);

    $time = new DrupalDateTime();
    $format_types = $this->dateFormatStorage->loadMultiple();
    $options = [];
    foreach ($format_types as $type => $type_info) {
      $format = $this->dateFormatter->format($time->getTimestamp(), $type);
      $options[$type] = $type_info->label() . ' (' . $format . ')';
    }

    $form['format_type'] = [
      '#type' => 'select',
      '#title' => $this->t('Date format'),
      '#description' => $this->t("Choose a format for displaying the date. Be sure to set a format appropriate for the field, i.e. omitting time for a field that only has a date."),
      '#options' => $options,
      '#default_value' => $this->getSetting('format_type'),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state) {
    $element = parent::formElement($items, $delta, $element, $form, $form_state);
    $date = $this->getFileDate($element);
    if ($date) {
      $element['#description'] .= '<h5 class="form-item__label">' . $this->t("This Version Uploaded On") . "</h5>";
      $element['#description'] .= "<p><strong><big>" . $date . "</big></strong></p>";
    }
    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = parent::settingsSummary();

    $date = new DrupalDateTime();
    $summary[] = $this->t('Format: @display', ['@display' => $this->formatTimestamp($date->getTimestamp())]);

    return $summary;
  }

  /**
   * Return a formatted datetime.
   *
   * @param array $element
   *   The file form element.
   */
  public function getFileDate($element) {
    $file = $this->getCurrentFile($element);
    if (!$file) {
      return NULL;
    }
    $timestamp = $file->get('created')->value;
    return $this->formatTimestamp($timestamp);
  }

  /**
   * Format the timestamp with the configured date format type.
   *
   * Riffing on datetime/Plugin/Field/FieldFormatter/DateTimeDefaultFormatter's
   * formatDate() method but we'd have to turn our timestamp into a date only to
   * turn it back into a timestamp, and that would be silly.
   */
  public function formatTimestamp($timestamp) {
    $format_type = $this->getSetting('format_type');
    return $this->dateFormatter->format($timestamp, $format_type);
  }

  /**
   * Load the current file, NULL if none.
   *
   * @param array $element
   *   The file form element.
   *
   * @return \Drupal\file\FileInterface|null
   *
   * @TODO find a better way than looking at the default value of the form,
   * there has got to be a better way right?
   */
  public function getCurrentFile($element) {
    $fid = $element['#default_value']['target_id'] ?? NULL;
    if ($fid) {
      return File::load($fid);
    }
    return NULL;
  }

}
